package ch.ethz.se.rpnc.ui;

//package ch.ethz.se.rpnc.view;

import java.io.BufferedReader;
import java.io.PrintWriter;
import java.util.Observer;

import ch.ethz.se.rpnc.ReversePolishNotationCalculator;
import ch.ethz.se.rpnc.instruct.CalculatorCommandType;
import ch.ethz.se.rpnc.instruct.CalculatorInstructCommand;
import ch.ethz.se.rpnc.instruct.CalculatorInstructOperand;
import ch.ethz.se.rpnc.instruct.CalculatorInstructOperator;
import ch.ethz.se.rpnc.instruct.ICalculatorInstruct;
import ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter;

public class CalculatorConsole implements ICalculatorView {

	private class CalculatorInstructConsoleExecutor extends
			CalculatorInstructVisitorAdapter {

		// Is the instruct considered processed after the visit?
		private boolean isInstructProcessed;

		/**
		 * Indicate whether the instruct has been processed.
		 * 
		 * @param isInstructProcessed
		 *            A flag.
		 */
		public void setInstructProcessed(boolean isInstructProcessed) {
			this.isInstructProcessed = isInstructProcessed;
		}

		/**
		 * Query for if the instruct has been processed.
		 * 
		 * @return A flag.
		 */
		public boolean isInstructProcessed() {
			return isInstructProcessed;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitCommand(ch.ethz.se.rpnc.instruct.CalculatorInstructCommand)
		 */
		@Override
		public void visitCommand(CalculatorInstructCommand instruct) {
			CalculatorConsole console = CalculatorConsole.this;
			setInstructProcessed(false);
			if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_DEUTSCH)
					|| instruct == CalculatorInstructCommand
							.getInstructCommand(CalculatorCommandType.CMD_ENGLISH)) {
				setCurrentLanguageByKey(instruct.getType());
				printCurrentLanguageName();
				printHelpMessage();
				setInstructProcessed(true);
			} else if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_HELP)) {
				printHelpMessage();
				setInstructProcessed(true);
			} else if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_PRINT)) {
				printCalculatorStatus();
				setInstructProcessed(true);
			} else if (instruct == CalculatorInstructCommand
					.getInstructCommand(CalculatorCommandType.CMD_QUIT)) {
				console.quit();
				setInstructProcessed(true);
			}
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitOperand(ch.ethz.se.rpnc.instruct.CalculatorInstructOperand)
		 */
		@Override
		public void visitOperand(
				CalculatorInstructOperand calculatorCommandOperand) {
			setInstructProcessed(false);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * ch.ethz.se.rpnc.instruct.visitor.CalculatorInstructVisitorAdapter
		 * #visitOperator(ch.ethz.se.rpnc.instruct.CalculatorInstructOperator)
		 */
		@Override
		public void visitOperator(
				CalculatorInstructOperator calculatorCommandOperator) {
			setInstructProcessed(false);
		}

		// Set current interface language by key.
		private void setCurrentLanguageByKey(CalculatorCommandType aType) {
			try {
				ResourceManager.setLanguage(aType);
			} catch (Exception e) {
				printMessage(e.getMessage());
			}

		}

		// Print calculator status.
		private void printCalculatorStatus() {
			printMessage(calculator.toString());
		}

		// Print current language name.
		private void printCurrentLanguageName() {
			printMessage(getLocalizedUIMessage(UIMessage.MSG_CURRENT_LANGUAGE)
					+ " " + ResourceManager.getLanguageName());
		}

		// Print help message.
		private void printHelpMessage() {
			printMessage(getLocalizedUIMessage(UIMessage.MSG_HELP_MESSAGE));
		}

	}

	// Input reader.
	private BufferedReader reader;

	// Output stream.
	private PrintWriter writer;

	// Command object to implement observer pattern.
	private CalculatorInstructEvent command;

	// Calculator as the model in MVC.
	private ReversePolishNotationCalculator calculator;

	// Instruct console executor.
	private CalculatorInstructConsoleExecutor executor;

	// Should the program quit?
	private boolean shouldQuit = false;

	/**
	 * Constructor.
	 * 
	 * @param aScanner
	 *            Source of input.
	 * @param aPrinter
	 *            Destination of output.
	 * @param aCalc
	 *            Calculator object.
	 */
	public CalculatorConsole(BufferedReader aReader, PrintWriter aWriter,
			ReversePolishNotationCalculator aCalc) {
		reader = aReader;
		writer = aWriter;

		calculator = aCalc;
		command = new CalculatorInstructEvent();
		executor = new CalculatorInstructConsoleExecutor();
	}

	/**
	 * Add a new observer to observe for new command.
	 * 
	 * @param o
	 *            New observer.
	 */
	public void addCommandObserver(Observer o) {
		command.addObserver(o);
	}

	/**
	 * Print message.
	 */
	public void printMessage(String str) {
		writer.println(str);
		writer.flush();
	}

	/**
	 * Signify the console to quit.
	 */
	public void quit() {
		shouldQuit = true;
	}

	/**
	 * Read, parse, and forward loop.
	 */
	public void run() {
		String inputCmd = "";

		while (!shouldQuit) {
			printPrompt();
			try {
				inputCmd = reader.readLine().trim();
			} catch (Exception e) {
				printMessage(e.getMessage());
				quit();
			}

			if (!shouldQuit) {
				ICalculatorInstruct instruct = parseCalculatorInstruct(inputCmd);
				if (instruct != null){
					try{
						processInstruct(instruct);
					}
					catch(Exception e){
						printMessage(e.getMessage());
					}
				}
				else{
					printMessage(getLocalizedUIMessage(UIMessage.MSG_INVALID_INPUT));
					printMessage("\t" + inputCmd);
				}
			}
		}

		printMessage(getLocalizedUIMessage(UIMessage.MSG_QUITTING_APP));
	}

	// Parse user input and construct corresponding calculator instructs.
	private ICalculatorInstruct parseCalculatorInstruct(String cmd) {
		ICalculatorInstruct instruct = null;
		if ((instruct = CalculatorInstructCommand.getInstructCommand(cmd)) != null) {
			return instruct;
		} else if ((instruct = CalculatorInstructOperator.getInstructOperator(cmd)) != null) {
			return instruct;
		} else {
			try {
				double d = Double.parseDouble(cmd);
				return new CalculatorInstructOperand(d);
			} catch (NumberFormatException nfe) {
			}
		}
		return null;
	}

	// Print command prompt.
	private void printPrompt() {
		writer.print("#");
		writer.flush();
	}

	// Process the calculator instructs up to the view's capability,
	// and pass the others to the controller.
	private void processInstruct(ICalculatorInstruct instruct) {
		assert (instruct != null);
		try {
			instruct.accept(executor);
		} catch (Exception e) {
			// Console executor should not raise any exception.
			printMessage(getLocalizedUIMessage(UIMessage.MSG_UNKNOWN_ERROR));
			assert (false);
		}
		if (!executor.isInstructProcessed()) {
			// Notify the controller of a new instruct.
			command.setInstruct(instruct);
		}
	}

	// Get the text in current language, according to aKey.
	// Return empty string if no text is found at the key.
	private String getLocalizedUIMessage(UIMessage msg) {
		String text = "";
		try {
			text = ResourceManager.getLocalizedUIMessage(msg);
		} catch (Exception e) {
			text = "";
		}
		return text;
	}

//	private String getLocalizedUICommand(CalculatorCommandType ct) {
//		String text = "";
//		try {
//			text = ResourceManager.getLocalizedUICommand(ct);
//		} catch (Exception e) {
//			text = "";
//		}
//		return text;
//	}

}
