package ch.ethz.se.rpnc.ui;

import java.io.BufferedReader;
import java.io.PrintWriter;

import ch.ethz.se.rpnc.ReversePolishNotationCalculator;

public class CalculatorConsole {

	// Input reader.
	private BufferedReader reader;

	// Output stream.
	private PrintWriter writer;

	// Calculator as the model in MVC.
	private ReversePolishNotationCalculator calculator;

	// Should the program quit?
	private boolean shouldQuit = false;

	/**
	 * Constructor.
	 * 
	 * @param aScanner
	 *            Source of input.
	 * @param aPrinter
	 *            Destination of output.
	 * @param aCalc
	 *            Calculator object.
	 */
	public CalculatorConsole(BufferedReader aReader, PrintWriter aWriter,
			ReversePolishNotationCalculator aCalc) {
		reader = aReader;
		writer = aWriter;

		calculator = aCalc;
	}

	/**
	 * Print message.
	 */
	public void printMessage(String str) {
		writer.println(str);
		writer.flush();
	}

	/**
	 * Signify the console to quit.
	 */
	public void quit() {
		shouldQuit = true;
	}

	/**
	 * Read, parse, and forward loop.
	 */
	public void run() {
		String inputCmd = "";

		while (!shouldQuit) {
			printPrompt();
			try {
				inputCmd = reader.readLine().trim();
			} catch (Exception e) {
				printMessage(e.getMessage());
				quit();
			}

			if (!shouldQuit && inputCmd.length() != 0) {
				parseInstructionAndExecute(inputCmd);
			}
		}

		printMessage(getLocalizedUIMessage(UIMessage.MSG_QUITTING_APP));
	}

	// Get the text in current language, according to aKey.
	// Return empty string if no text is found at the key.
	private String getLocalizedUIMessage(UIMessage msg) {
		String text = "";
		try {
			text = ResourceManager.getLocalizedUIMessage(msg);
		} catch (Exception e) {
			text = "";
		}
		return text;
	}

	private void parseInstructionAndExecute(String ins) {
		ins = ins.trim();

		try {
			if (ins.equals("+")) {
				calculator.pushPlusOperator();
			} else if (ins.equals("-")) {
				calculator.pushMinusOperator();
			} else if (ins.equals("*")) {
				calculator.pushTimesOperator();
			} else if (ins.equals("/")) {
				calculator.pushDivideByOperator();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_DEUTSCH))) {
				setCurrentLanguageByKey(CalculatorCommandType.CMD_DEUTSCH);
				printCurrentLanguageName();
				printHelpMessage();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_ENGLISH))) {
				setCurrentLanguageByKey(CalculatorCommandType.CMD_ENGLISH);
				printCurrentLanguageName();
				printHelpMessage();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_CLEAR))) {
				calculator.clearStack();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_HELP))) {
				printHelpMessage();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_PRINT))) {
				printCalculatorStatus();
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_EVALUATE))) {
				try {
					calculator.evaluateStack();
				} catch (Exception e) {
					printMessage(e.getMessage());
				}
			} else if (ins.equals(ResourceManager
					.getLocalizedUICommand(CalculatorCommandType.CMD_QUIT))) {
				quit();
			} else {
				try {
					double d = Double.parseDouble(ins);
					calculator.pushOperand(d);
				} catch (NumberFormatException nfe) {
					printMessage(getLocalizedUIMessage(UIMessage.MSG_INVALID_INPUT));
					printMessage("\t" + ins);
				}
			}
		} catch (Exception e) {
			printMessage(getLocalizedUIMessage(UIMessage.MSG_UNKNOWN_ERROR));
		}

	}

	// Print calculator status.
	private void printCalculatorStatus() {
		printMessage(calculator.toString());
	}

	// Print current language name.
	private void printCurrentLanguageName() {
		printMessage(getLocalizedUIMessage(UIMessage.MSG_CURRENT_LANGUAGE)
				+ " " + ResourceManager.getLanguageName());
	}

	// Print help message.
	private void printHelpMessage() {
		printMessage(getLocalizedUIMessage(UIMessage.MSG_HELP_MESSAGE));
	}

	// Print command prompt.
	private void printPrompt() {
		writer.print("#");
		writer.flush();
	}

	// Set current interface language by key.
	private void setCurrentLanguageByKey(CalculatorCommandType aType) {
		try {
			ResourceManager.setLanguage(aType);
		} catch (Exception e) {
			printMessage(e.getMessage());
		}

	}

}
